/*	$Id: coda_nt.h,v 1.23 2005/08/22 21:48:34 phil Exp $ */

/* BLURB gpl

                           Coda File System
                              Release 6

          Copyright (c) 1987-2005 Carnegie Mellon University
                  Additional copyrights listed below

This  code  is  distributed "AS IS" without warranty of any kind under
the terms of the GNU General Public Licence Version 2, as shown in the
file  LICENSE.  The  technical and financial  contributors to Coda are
listed in the file CREDITS.

*/


/* 
 * NT support for Coda
 * Peter Braam, 9/1996
 *
 */

#ifndef _CODA_NT_H
#define _CODA_NT_H

#include <stdarg.h>
#include <ntddk.h>
#include "errno.h"
#include "dllist.h"
#include "wrapper.h"        // OSR Wrapper Defines
#include "wrapper-ioctl.h"  // OSR Wrapper IOCTL/FSCTL functions

#include "debug.h"
#include "codafsctl.h"

/* string support */

extern int strnlen(const char *, int);
extern void * memscan(void *,int,int);
int sprintf(char * buf, const char *fmt, ...);
int vsprintf(char *buf, const char *fmt, va_list args);

/* locks */

/* upcalls */
int venus_rootfid(CodaFid *);
int venus_getattr(CodaFid *fid, struct coda_vattr *attr, int uid);
int venus_setattr(CodaFid *fid, struct coda_vattr *attr, int uid);
int venus_lookup(CodaFid *fid, const char *name, int length,
		 int * type, CodaFid *resfid, int uid);
int venus_close(CodaFid *, int flags, int uid);
int venus_open_by_path(CodaFid *fid, int flags, char *cfname, int uid);
int venus_access(CodaFid *fid, int mask, int uid);
int venus_pioctl(CodaFid *fid, 
		 struct PioctlData *pdata, int pdatasize,
		 void* outbuf, int *outbufsize, int uid);
int venus_create(CodaFid *dirfid, 
		 const char *name, int length, int excl, int mode, int rdev,
		 CodaFid *newfid, struct coda_vattr *attrs, int uid);
int venus_mkdir(CodaFid *dirfid, const char *name, int length, 
		CodaFid *newfid, struct coda_vattr *attrs, int uid);
int venus_remove(CodaFid *dirfid, const char *name, int length,
		 int uid);
int venus_rmdir(CodaFid *dirfid, const char *name, int length, int uid);
int venus_rename(CodaFid *old_fid, const char *old_name, 
		 size_t old_length, 
		 CodaFid *new_fid, const char *new_name,
		 size_t new_length, int uid);
int venus_statfs(struct coda_statfs *sfs);

 
/* osr interface prototypes */
//
// Entry points from the wrapper
//
NTSTATUS coda_mount(IN PVOID MediaHandle, OUT PFS_VOL_HANDLE CodaVolumep,
		    OUT PFS_FILE_HANDLE CodaRootDirHandle);
NTSTATUS coda_unmount(IN FS_VOL_HANDLE CodaVolume);
NTSTATUS coda_verify(IN FS_VOL_HANDLE CodaVolume); 
NTSTATUS coda_get_volume_attributes(IN FS_VOL_HANDLE CodaVolume,
				    IN OUT PFS_VOL_ATTRIBUTES Attributes,
				    IN OUT PULONG BufferSize);
NTSTATUS coda_getattr(IN FS_FILE_HANDLE FileHandle,
		      IN OUT PFS_FILE_ATTRIBUTES FileAttributes);
NTSTATUS coda_setattr(IN FS_FILE_HANDLE FileHandle,
		      IN OUT PFS_FILE_ATTRIBUTES FileAttributes);
NTSTATUS coda_set_length(IN FS_FILE_HANDLE FileHandle,
			 IN LARGE_INTEGER AllocationSize,
			 IN LARGE_INTEGER DataSize);
NTSTATUS coda_release(IN FS_FILE_HANDLE FileHandle);
NTSTATUS coda_delete2(IN FS_FILE_HANDLE FileHandle,
		      IN FS_FILE_HANDLE ParentHandle,
		      IN PWSTR FileName);
NTSTATUS coda_fsctrl(IN FS_FILE_HANDLE FileHandle, IN ULONG ControlCode,
		    IN PVOID InputBuffer, IN ULONG InputBufferSize,
		    IN PVOID OutputBuffer, IN ULONG OutputBufferSize,
		    OUT PULONG BytesWritten, OUT PULONG BytesNeeded);
// NTSTATUS CodaUncRoot(OUT PFS_FILE_HANDLE DirectoryHandle);
NTSTATUS coda_lookup(IN FS_FILE_HANDLE DirectoryHandle, IN PWCHAR FileName,
		     IN BOOLEAN CaseSensitive, OUT PFS_FILE_HANDLE FileHandle,
		     OUT PULONG Type);
NTSTATUS coda_readdir(IN FS_FILE_HANDLE DirectoryHandle,
		      IN OUT PULONG DirectoryOffset,
		      IN PFS_DIRECTORY_ENTRY OutputBuffer,
		      IN ULONG OutputBufferSize);
NTSTATUS coda_read(IN FS_FILE_HANDLE FileHandle, IN LARGE_INTEGER FileOffset,
		   IN PMDL Buffer, IN ULONG BufferSize, OUT PULONG BytesRead);
NTSTATUS coda_write(IN FS_FILE_HANDLE FileHandle, IN LARGE_INTEGER FileOffset,
		    IN PMDL Buffer, IN ULONG BufferSize,
		    OUT PULONG BytesWritten);

NTSTATUS coda_create(IN FS_FILE_HANDLE DirectoryHandle, IN PWCHAR FileName,
		    IN PWCHAR ShortFileName, IN ULONG FileType,
		    OUT PFS_FILE_HANDLE NewFileHandle);

NTSTATUS coda_ioctl(IN FS_FILE_HANDLE FileHandle, IN ULONG ControlCode,
		    IN PVOID InputBuffer, IN ULONG InputBufferSize,
		    IN PVOID OutputBuffer, IN ULONG OutputBufferSize,
		    OUT PULONG BytesWritten, OUT PULONG BytesNeeded);

NTSTATUS coda_rename2(IN FS_FILE_HANDLE SourceDir,
		      IN FS_FILE_HANDLE TargetDir,
		      IN FS_FILE_HANDLE SourceHandle,
		      IN PWSTR SourceName,
		      IN PWSTR TargetName,
		      IN PWSTR AltTargetName,
		      OUT PFS_FILE_HANDLE TargetHandle);

NTSTATUS coda_updshare (IN FS_FILE_HANDLE FileHandle, IN ULONG Access);

NTSTATUS coda_remshare (IN FS_FILE_HANDLE FileHandle, IN ULONG Access);

NTSTATUS coda_access(IN FS_FILE_HANDLE FileHandle,
		     IN OUT PACCESS_STATE pAccessState);

//
// pseudo entry point!
//
NTSTATUS coda_pioctl (IN struct PioctlData *buf, IN ULONG bufsize,
		      IN PVOID outbuf, IN OUT PULONG outsize);

NTSTATUS coda_downcall(IN int opcode, IN union outputArgs * out);

//
// NT Support functions
//
BOOLEAN RtlCreateUnicodeStringFromAsciiz(UNICODE_STRING DestinationString,
					 PCSZ SourceString);
VOID RtlSecondsSince1970ToTime (ULONG ElapsedSeconds,PLARGE_INTEGER Time);
BOOLEAN  RtlTimeToSecondsSince1970 (PLARGE_INTEGER Time,
				    PULONG ElapsedSeconds);

NTSTATUS NullDiscardIrpCompletion(PDEVICE_OBJECT DeviceObject,
                                  PIRP Irp,
                                  PVOID Context);
/* debugging constants */

extern char *Coda_DriverName;

/* version information */
#define Ver_pre 'A'         // A = Alpha, B = Beta, V = Omega
#define Ver_maj 1
#define Ver_min 1

#define MAX(a,b)  ( (a) > (b) ) ? (a) : (b)
#define MIN(a,b)  ( (a) < (b) ) ? (a) : (b)

#define FID(fid)  (fid)->opaque[0], (fid)->opaque[1], \
		  (fid)->opaque[2], (fid)->opaque[3]

//
// Define an ASSERT macro over which we have control
//
#undef  ASSERT

#if ASSERTS_PRINT
    #define ASSERT(x) if(!(x)) DbgPrint("\n^^^^^^\n\t\tAssert failed at %s line # %d.\n^^^^^^^^^\n\n",__FILE__,__LINE__)
#else
    #define ASSERT(x)
#endif

#define coda_malloc(len)	ExAllocatePool(NonPagedPool, (len))
#define coda_free(p)		ExFreePool(p)


/* Token manipulation */

#define TOKEN_SOURCE_LENGTH 8
typedef struct _TOKEN_SOURCE {
        CHAR SourceName[TOKEN_SOURCE_LENGTH];
	LUID SourceIdentifier;
} TOKEN_SOURCE, *PTOKEN_SOURCE;

typedef struct _TOKEN_CONTROL {
        LUID TokenId;
	LUID AuthenticationId;
	LUID ModifiedId;
	TOKEN_SOURCE TokenSource;
} TOKEN_CONTROL, *PTOKEN_CONTROL;


typedef struct _SECURITY_CLIENT_CONTEXT {
         SECURITY_QUALITY_OF_SERVICE SecurityQos;
	 PACCESS_TOKEN ClientToken;
	 BOOLEAN DirectlyAccessClientToken;
	 BOOLEAN DirectAccessEffectiveOnly;
	 BOOLEAN ServerIsRemote;
	 TOKEN_CONTROL ClientTokenControl;
} SECURITY_CLIENT_CONTEXT, *PSECURITY_CLIENT_CONTEXT;




NTKERNELAPI
NTSTATUS
IoCreateFile(
    OUT PHANDLE FileHandle,
    IN ACCESS_MASK DesiredAccess,
    IN POBJECT_ATTRIBUTES ObjectAttributes,
    OUT PIO_STATUS_BLOCK IoStatusBlock,
    IN PLARGE_INTEGER AllocationSize OPTIONAL,
    IN ULONG FileAttributes,
    IN ULONG ShareAccess,
    IN ULONG Disposition,
    IN ULONG CreateOptions,
    IN PVOID EaBuffer OPTIONAL,
    IN ULONG EaLength,
    IN CREATE_FILE_TYPE CreateFileType,
    IN PVOID ExtraCreateParameters OPTIONAL,
    IN ULONG Options
    );

NTKERNELAPI
NTSTATUS
SeCreateClientSecurity (
    IN PETHREAD ClientThread,
    IN PSECURITY_QUALITY_OF_SERVICE ClientSecurityQos,
    IN BOOLEAN RemoteSession,
    OUT PSECURITY_CLIENT_CONTEXT ClientContext
    );


NTKERNELAPI
BOOLEAN
IoIsSystemThread(
    IN PETHREAD Thread
    );


// Security routine defined in misc.c

int GetNtUID(void);

// Security definitions needed.  From OSR.

// Some NT declarations we will be using a bit later in this file to
// extract the SID of the caller.  All this information is derived
// from public sources (look at winnt.h for the defines for instance)
// 

typedef enum _TOKEN_INFORMATION_CLASS {
    TokenUser = 1,
    TokenGroups,
    TokenPrivileges,
    TokenOwner,
    TokenPrimaryGroup,
    TokenDefaultDacl,
    TokenSource,
    TokenType,
    TokenImpersonationLevel,
    TokenStatistics
} TOKEN_INFORMATION_CLASS, *PTOKEN_INFORMATION_CLASS;

#define TOKEN_QUERY (0x0008)
#define TOKEN_READ  (STANDARD_RIGHTS_READ|TOKEN_QUERY)

typedef struct _SID_IDENTIFIER_AUTHORITY {
    UCHAR  Value[6];
} SID_IDENTIFIER_AUTHORITY, *PSID_IDENTIFIER_AUTHORITY;

typedef struct _SID {
   UCHAR  Revision;
   UCHAR  SubAuthorityCount;
   SID_IDENTIFIER_AUTHORITY IdentifierAuthority;
   ULONG SubAuthority[ANYSIZE_ARRAY];
} SID, *PISID;

typedef struct _SID_AND_ATTRIBUTES {
    PSID Sid;
    ULONG Attributes;
} SID_AND_ATTRIBUTES, * PSID_AND_ATTRIBUTES;

typedef struct {
    SID_AND_ATTRIBUTES    User;
} TOKEN_USER, *PTOKEN_USER;

//
// NT entry points for obtaining thread security information
//

NTSTATUS ZwOpenThreadToken (HANDLE, ACCESS_MASK, BOOLEAN, PHANDLE);
NTSTATUS ZwOpenProcessToken (HANDLE, ACCESS_MASK, PHANDLE);
NTSTATUS ZwQueryInformationToken (HANDLE, TOKEN_INFORMATION_CLASS, PVOID,
				  ULONG, PULONG);

// Wrapper lock

ERESOURCE CodaWrapperLock;
#define GrabLock() ExAcquireResourceExclusiveLite(&CodaWrapperLock, TRUE);
#define DropLock() ExReleaseResourceLite(&CodaWrapperLock);

// Misc defines

#define RunningAtPassiveLevel (KeGetCurrentIrql() == PASSIVE_LEVEL)

// Global variables ... defined in coda.c

// Global data for work queues
extern PDEVICE_OBJECT coda_device_object;

// Global data for "volume" management

extern struct coda_ntvolume coda_nt_volume;

#endif _CODA_NT_H


